@@ -3,9 +3,9 @@
  * Horde_Mobile_Renderer:: output module for simple HTML and
  * Imode/Avantgo/similar devices.
  *
- * $Horde: framework/Mobile/Mobile/Renderer/html.php,v 1.28.10.4 2005/09/30 02:51:21 selsky Exp $
+ * $Horde: framework/Mobile/Mobile/Renderer/html.php,v 1.28.10.11 2009/01/06 15:23:25 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -35,7 +35,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
      *
      * @param Horde_Mobile $deck  The deck to render.
      */
-    function render(&$deck)
+    function render($deck)
     {
         if ($deck->_debug) {
             header('Content-Type: text/plain; charset=' . NLS::getCharset());
@@ -108,7 +108,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         echo '</body></html>';
     }
 
-    function _renderCard(&$card)
+    function _renderCard($card)
     {
         $name = $card->get('name') ? ' name="' . $this->escape($card->get('name')) . '"' : '';
         printf('<a%s>%s</a>', $name, $card->get('title'));
@@ -125,7 +125,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         }
     }
 
-    function _renderLink(&$link)
+    function _renderLink($link)
     {
         if ($link->get('title') &&
             !$this->isBrowser('avantgo') &&
@@ -152,7 +152,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
                $this->escape($link->get('label')));
     }
 
-    function _renderLinkset(&$linkset)
+    function _renderLinkset($linkset)
     {
         if (count($linkset->_elements)) {
             echo '<ol>';
@@ -165,7 +165,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         }
     }
 
-    function _renderText(&$element)
+    function _renderText($element)
     {
         foreach ($element->_attributes as $attribute) {
             echo '<' . $attribute . '>';
@@ -183,14 +183,23 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         }
     }
 
-    function _renderForm(&$form)
+    function _renderImage($image)
+    {
+        $attributes = '';
+        foreach ($image->_attributes as $attribute => $value) {
+            $attributes .= sprintf(' %s="%s"', $attribute, $value);
+        }
+        printf('<img src="%s.png"%s />', $image->_src, $attributes);
+    }
+
+    function _renderForm($form)
     {
         printf('<form action="%s" method="%s">', $form->get('url'), $form->get('method'));
         parent::_renderForm($form);
         echo '</form>';
     }
 
-    function _renderInput(&$input)
+    function _renderInput($input)
     {
         $type = 'type="' . $input->get('type') . '"';
         $size = $input->get('size') ? sprintf('size="%d"', $input->get('size')) : '';
@@ -210,7 +219,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
                $this->escape($input->get('name')), $this->escape($input->get('value')), $size, $maxlength, $mode);
     }
 
-    function _renderTextarea(&$textarea)
+    function _renderTextarea($textarea)
     {
         if ($this->isBrowser('imode')) {
             $mode = sprintf(' istyle="%d"', $this->mode);
@@ -220,31 +229,34 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
             $mode = '';
         }
 
-        $wrap = '';
-        if (!$this->isBrowser('imode') && !$this->isBrowser('mml')) {
-            $wrap = ' wrap="virtual"';
-        }
-
-        printf('%s<br /><textarea name="%s" rows="%s" cols="%s"%s%s>%s</textarea>',
+        printf('%s<br /><textarea name="%s" rows="%s" cols="%s"%s>%s</textarea>',
                $this->escape($textarea->get('label')), $textarea->get('name'), $textarea->get('rows'),
-               $textarea->get('cols'), $mode, $wrap, $textarea->get('value'));
+               $textarea->get('cols'), $mode, $textarea->get('value'));
     }
 
-    function _renderSelect(&$select)
+    function _renderSelect($select)
     {
-        echo '<select name="' . $select->get('name') . '" size="1">';
+        $name = $this->escape($select->get('name'));
+        echo '<label for="' . $name . '">';
+        if ($label = $select->get('label')) {
+            echo $this->escape($label) . ' ';
+        }
+        echo '<select id="' . $name . '" name="' . $name . '" size="1">';
+
+        $htmlchars = $select->get('htmlchars');
         foreach ($select->_options as $val) {
             if ($val['value'] == $select->_value) {
                 $sel = ' selected="selected"';
             } else {
                 $sel = '';
             }
-            echo '<option' . $sel . ' value="' . $val['value'] . '">' . $this->escape($val['label']) . '</option>';
+            $label = $htmlchars ? $val['label'] : $this->escape($val['label']);
+            echo '<option' . $sel . ' value="' . $this->escape($val['value']) . '">' . $label . '</option>';
         }
-        echo '</select>';
+        echo '</select></label>';
     }
 
-    function _renderRadio(&$radio)
+    function _renderRadio($radio)
     {
         foreach ($radio->_buttons as $val) {
             $sel = ($val['value'] == $radio->_value) ? ' checked="checked"' : '';
@@ -254,28 +266,43 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         }
     }
 
-    function _renderCheckbox(&$checkbox)
+    function _renderCheckbox($checkbox)
     {
         $state = $checkbox->isChecked() ? ' checked="checked"' : '';
-        printf('<input type="checkbox" name="%s"%s value="%s" /> %s<br />',
+        printf('<label for="%1$s"><input type="checkbox" name="%1$s" id="%1$s"%2$s value="%3$s" /> %4$s</label><br />',
                $checkbox->get('name'), $state, $checkbox->get('value'),
                $this->escape($checkbox->get('label')));
     }
 
-    function _renderSubmit(&$submit)
+    function _renderSubmit($submit)
     {
         $name = !empty($submit->_name) ? ' name="' . $submit->_name . '"' : '';
         printf('<input type="submit"%s value="%s" /><br />',
                $name, $this->escape($submit->_label));
     }
 
-    function _renderHidden(&$hidden)
+    function _renderHidden($hidden)
     {
         printf('<input type="hidden" name="%s" value="%s" />',
                $hidden->get('name'), $hidden->get('value'));
     }
 
-    function _renderTable(&$table)
+    function _renderDl($dl)
+    {
+        echo '<dl>';
+
+        parent::_renderDl($dl);
+
+        // Terminate Dl.
+        if ($this->isBrowser('mml')) {
+            // MML has problems with the clear attribute.
+            echo '</dl><br />';
+        } else {
+            echo '</dl><br clear="all" />';
+        }
+    }
+
+    function _renderTable($table)
     {
         $border = $table->get('border');
         $padding = $table->get('padding');
@@ -300,11 +327,11 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
             echo '</table><br />';
         } else {
             // MML has problems with the clear attribute.
-            echo '</table><br clear="all">';
+            echo '</table><br clear="all" />';
         }
     }
 
-    function _renderPhone(&$phone)
+    function _renderPhone($phone)
     {
         if ($this->isBrowser('imode')) {
             // Create phoneto: link for i-Mode.
@@ -320,7 +347,7 @@ class Horde_Mobile_Renderer_html extends Horde_Mobile_Renderer {
         }
     }
 
-    function _renderRule(&$rule)
+    function _renderRule($rule)
     {
         $width = $rule->get('width');
         $size = $rule->get('size');
